%global shver 3
%global maven_group_id tw.edu.ntu.csie
%global pom_file_version 3.30
%global octpkg %{name}
%global release_date 2023-02-28
%global cpp_std c++17
%global upver %%(tr -d . <<< %{version})

%bcond_with java
%bcond_with octave
%bcond_with python

Summary:        A Library for Support Vector Machines
Name:           libsvm
Version:        3.31
Release:        5%{?dist}
License:        BSD-3-Clause
URL:            https://www.csie.ntu.edu.tw/~cjlin/libsvm/
Source0:        https://github.com/cjlin1/libsvm/archive/refs/tags/v%{upver}.tar.gz
Source1:        https://www.csie.ntu.edu.tw/~cjlin/libsvm/log
Source2:        https://www.csie.ntu.edu.tw/~cjlin/papers/guide/guide.pdf
Source3:        libsvm-svm-toy-qt.desktop
Source4:        LibSVM-svm-toy-48.png
# java interface
Source5:        https://repo1.maven.org/maven2/tw/edu/ntu/csie/%{name}/%{pom_file_version}/%{name}-%{pom_file_version}.pom
# octave interface
Source6:        libsvm.INDEX
Source7:        libsvm.CITATION
Source8:        libsvm.DESCRIPTION
Patch3000:      %{name}.packageMain.patch
Patch3001:      %{name}.javaDir.patch
Patch3002:      %{name}.toolsDir.patch
Patch3003:      %{name}.svm-toy-qt5.patch
Patch3004:      %{name}.matlab.patch

%description
Libsvm is a simple, easy-to-use, and efficient software for SVM classification and regression. 
It solves C-SVM classification, nu-SVM classification, one-class-SVM, epsilon-SVM regression, 
and nu-SVM regression. It provides an automatic model selection tool for C-SVM classification.

%package devel
Summary:        Development files for libsvm in C, C++ and Java
BuildRequires:  make gcc-c++
Requires:       %{name} = %{version}-%{release}

%description    devel
This package provides header and object files for libsvm in C, C++ and Java.

%if %{with python}
%package -n     python3-%{name}
Summary:        Python3 tools and interfaces for libsvm
BuildRequires:  python3-devel
# easy.py require
Requires:       gnuplot

%description -n python3-%{name}
This package provides python3 tools and interfaces for libsvm.
%endif

%if %{with java}
%package        java
Summary:        Java tools and interfaces for libsvm
BuildRequires:  java-devel javapackages-tools
BuildRequires:  maven-local
Requires:       javapackages-tools
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description    java
This package provides java tools and interfaces for libsvm.

%package        javadoc
Summary:        Javadoc for libsvm
BuildRequires:  java-devel
BuildRequires:  javapackages-tools
BuildArch:      noarch
Requires:       %{name}-java = %{version}-%{release}

%description    javadoc
This package provides javadoc for libsvm.
%endif

%if %{with octave}
%package -n     octave-%{name}
Summary:        Octave interface to libsvm
BuildRequires:  octave-devel
Requires:       %{name} = %{version}-%{release}
Requires:       octave

%description -n octave-%{name}
This package provides octave interface for libsvm.
%endif

%package        svm-toy-qt
Summary:        QT version of svm-toy (libsvm demonstration program)
BuildRequires:  desktop-file-utils
BuildRequires:  pkgconfig
BuildRequires:  qt5-qtbase-devel
Requires:       %{name} = %{version}-%{release}
Requires:       hicolor-icon-theme

%description    svm-toy-qt
This package provides a libsvm demonstration program which has a qt-GUI to display the 
derived separating hyperplane.

%prep
%autosetup -p0 -n %{name}-%{upver}
cp -p %{SOURCE1} ChangeLog
cp -p %{SOURCE2} %{SOURCE3} .
cp -p %{SOURCE4} %{name}-svm-toy-qt-48.png

%if %{with java}
cp -p %{SOURCE5} pom.xml

%pom_remove_parent

# update the pom file from version 3.30
%pom_xpath_set '/pom:project/pom:version' %{version}

%mvn_file %{maven_group_id}:%{name} %{maven_group_id}/%{name}
%endif

sed -i.orig 's/\r//' FAQ.html
touch -r FAQ.html.orig FAQ.html
rm FAQ.html.orig

%if %{with python}
%py3_shebang_fix tools
for p in python/libsvm/{commonutil,svm,svmutil}.py; do
    sed -i.orig '1,+1d' $p
    touch -r $p.orig $p
    rm $p.orig
done
%endif

%if %{with python}
%generate_buildrequires
cd python
%pyproject_buildrequires
%endif

%build
# build library
make all RPM_CFLAGS='%{build_cflags}' LIBDIR='%{_libdir}' CPP_STD='%{cpp_std}'

%if %{with java}
# build Java interface
%mvn_artifact pom.xml java/%{name}.jar
make -C java all javadoc
%endif

%if %{with octave}
# build octave interface
cd matlab
octave -H -q --no-window-system --no-site-file << EOF
make
EOF
cd -
%endif

%if %{with python}
# build python interface
cd python
%pyproject_wheel
cd -
%endif

%install
%make_install LIBDIR='%{_libdir}' LIBSVM_VER='%{version}' CPP_STD='%{cpp_std}'
mkdir -p %{buildroot}%{_datadir}/icons/hicolor/48x48/apps/
cp -p %{name}-svm-toy-qt-48.png %{buildroot}%{_datadir}/icons/hicolor/48x48/apps/
mkdir -p %{buildroot}%{_datadir}/applications
cp -p %{name}-svm-toy-qt.desktop %{buildroot}%{_datadir}/applications

ln -s %{name}.so.%{shver} %{buildroot}%{_libdir}/%{name}.so

%if %{with python}
# install python 
cd python
%pyproject_install
%pyproject_save_files libsvm
cd -
cd tools
for p in *.py; do
    install -p -m 755 $p %{buildroot}%{_bindir}/svm-$p
done
cd -
%endif

%if %{with java}
# install java
make -C java install JAVA_TARGET_DIR='%{buildroot}%{_javadir}'
mkdir -p  %{buildroot}%{_javadocdir}/%{name}
cp -p -R java/docs/* %{buildroot}%{_javadocdir}/%{name}

%mvn_install
%endif

%if %{with octave}
# install octave
# octave refuses to load the *.mex files from octpkglibdir.
# It will only load them if they are in octpkgdir.
mkdir -p %{buildroot}%{octpkgdir}/packinfo
cp -p matlab/*.mex %{buildroot}%{octpkgdir}
cp -p COPYRIGHT %{buildroot}%{octpkgdir}/packinfo/COPYING
cp -p %{SOURCE6} %{buildroot}%{octpkgdir}/packinfo/INDEX
cp -p %{SOURCE7} %{buildroot}%{octpkgdir}/packinfo/CITATION
sed 's/@VERSION@/%{version}/;s/@DATE@/%{release_date}/' %{SOURCE8} \
    > %{buildroot}%{octpkgdir}/packinfo/DESCRIPTION
cat > %{buildroot}%{octpkgdir}/packinfo/on_uninstall.m << EOF
function on_uninstall (desc)
  error ('Can not uninstall %%s', desc.name);
endfunction
EOF
%endif

# install desktop files
desktop-file-install --delete-original \
  --dir=%{buildroot}%{_datadir}/applications \
  %{buildroot}%{_datadir}/applications/%{name}-svm-toy-qt.desktop

mv python/README python/README-Python
mv tools/README tools/README-Tools
cp -p README java/README-Java

%if %{with octave}
%post -n octave-%{name}
%octave_cmd pkg rebuild

%preun -n octave-%{name}
%octave_pkg_preun

%postun -n octave-%{name}
%octave_cmd pkg rebuild
%endif

%files
%license COPYRIGHT
%doc FAQ.html ChangeLog guide.pdf
%{_bindir}/svm-predict
%{_bindir}/svm-scale
%{_bindir}/svm-train
%{_libdir}/%{name}.so.3
%dir %{_datadir}/%{name}/
%{_datadir}/%{name}/examples

%files devel
%doc README
%{_includedir}/%{name}/
%{_libdir}/%{name}.so

%if %{with octave}
%files -n octave-%{name}
%{octpkgdir}/
%endif

%if %{with python}
%files -n python3-%{name} -f %{pyproject_files}
%doc python/README-Python tools/README-Tools
%{_bindir}/svm-*.py
%endif

%if %{with java}
%files java -f .mfiles
%doc java/README-Java
%{_javadir}/%{name}.jar

%files javadoc
%{_javadocdir}/%{name}/
%endif

%files svm-toy-qt
%doc README
%{_bindir}/svm-toy-qt
%{_datadir}/icons/hicolor/48x48/apps/%{name}-svm-toy-qt-48.png
%{_datadir}/applications/*%{name}-svm-toy-qt.desktop

%changelog
* Wed Jun 11 2025 bbrucezhang <bbrucezhang@tencent.com> - 3.31-5
- Rebuilt for loongarch64

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.31-4
- Rebuilt for loongarch release

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.31-3
- Rebuilt for OpenCloudOS Stream 23.09

* Mon May 8 2023 Wang Guodong <gordonwwang@tencent.com> - 3.31-2
- modify prompt information

* Wed Apr 26 2023 Wang Guodong <gordonwwang@tencent.com> - 3.31-1
- initial build
